from django.conf.urls.defaults import *

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('project.zoo.views',
    # Example:
    (r'^$', 'index'),
)



"""
nose plugin for easy testing of django projects and apps. Sets up a test
database (or schema) and installs apps from test settings file before tests
are run, and tears the test database (or schema) down after all tests are run.
"""

from __future__ import with_statement

import logging
import os
import re
import sys

import nose.case
from nose.plugins import Plugin

# Force settings.py pointer
# search the current working directory and all parent directories to find
# the settings file
from nose.importer import add_path
if 'DJANGO_SETTINGS_MODULE' not in os.environ:
    os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

logger = logging.getLogger('nose.plugins.nosedjango')

NT_ROOT = re.compile(r"^[a-zA-Z]:\\$")


def get_settings_path(settings_module):
    '''
    Hunt down the settings.py module by going up the FS path
    '''
    cwd = os.getcwd()
    settings_filename = '%s.py' % (
        settings_module.split('.')[-1]
    )
    while cwd:
        if settings_filename in os.listdir(cwd):
            break
        cwd = os.path.split(cwd)[0]
        if os.name == 'nt' and NT_ROOT.match(cwd):
            return None
        elif cwd == '/':
            return None
    return cwd


def _dummy(*args, **kwargs):
    """Dummy function that replaces the transaction functions"""
    return


class ContextDecorator(object):
    """
    A base class that enables a context manager to also be used as a decorator.
    """
    def __init__(self, *args, **kwargs):
        return

    def __enter__(self):
        return

    def __exit__(self, exc_type, exc_value, traceback):
        return

    def __call__(self, func):
        def inner(*args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return inner


def _dummy_context_manager(*args, **kwargs):
    return ContextDecorator(*args, **kwargs)


class NoseDjango(Plugin):
    """
    Enable to set up django test environment before running all tests, and
    tear it down after all tests.

    Note that your django project must be on PYTHONPATH for the settings file
    to be loaded. The plugin will help out by placing the nose working dir
    into sys.path if it isn't already there, unless the -P
    (--no-path-adjustment) argument is set.
    """
    name = 'django'

    DJANGO_1_4 = (1, 4)
    DJANGO_1_7 = (1, 7)
    DJANGO_1_8 = (1, 8)

    def __init__(self):
        Plugin.__init__(self)
        self.nose_config = None
        self.django_plugins = []

        self._loaded_test_fixtures = []
        self._num_fixture_loads = 0
        self._num_flush_calls = 0
        self._num_syncdb_calls = 0

    def __del__(self):
        self.restore_transaction_support()

    @property
    def transaction(self):
        from django.db import transaction
        if self.django_version < self.DJANGO_1_7:
            transaction.set_autocommit = _dummy
            transaction.get_autocommit = _dummy
        elif self.django_version > self.DJANGO_1_7:
            transaction.enter_transaction_management = _dummy
            transaction.leave_transaction_management = _dummy
            transaction.managed = _dummy
        return transaction

    @property
    def TestCase(self):
        if hasattr(self, '_TestCase'):
            return self._TestCase
        import django.test.testcases
        self._TestCase = django.test.testcases.TestCase
        return self._TestCase

    @property
    def django_version(self):
        from django import VERSION
        return VERSION

    def commit(self):
        if self.django_version < self.DJANGO_1_7:
            self.transaction.commit()

    def rollback(self):
        if self.django_version < self.DJANGO_1_7:
            self.transaction.rollback()

    def enable_autocommit(self):
        self.old_autocommit = self.transaction.get_autocommit()
        self.transaction.set_autocommit(True)

    def restore_autocommit(self):
        if not hasattr(self, 'old_autocommit'):
            # An error occurred, early exit so we don't squash the error
            # message.
            return

        self.transaction.set_autocommit(self.old_autocommit)

    def enter_atomics(self):
        if self.django_version < self.DJANGO_1_7:
            return
        self.TestCase.cls_atomics = self.TestCase._enter_atomics()

    def exit_atomics(self):
        if self.django_version < self.DJANGO_1_7:
            return
        if not hasattr(self.TestCase, 'cls_atomics'):
            # An error occurred, early exit so we don't squash the error
            # message.
            return
        try:
            self.TestCase._rollback_atomics(self.TestCase.cls_atomics)
        except Exception:
            # If we can't roll back atomics, it means we crashed somewhere in
            # setup. Don't hide that error by eroding here too.
            pass

    def store_original_transaction_methods(self):
        self.orig_commit = self.transaction.commit
        self.orig_rollback = self.transaction.rollback
        self.orig_savepoint_commit = self.transaction.savepoint_commit
        self.orig_savepoint_rollback = self.transaction.savepoint_rollback
        self.orig_enter = self.transaction.enter_transaction_management
        self.orig_leave = self.transaction.leave_transaction_management
        if self.django_version > self.DJANGO_1_7:
            self.orig_atomic = self.transaction.atomic

    def disable_transaction_support(self):
        self.transaction.commit = _dummy
        self.transaction.rollback = _dummy
        self.transaction.savepoint_commit = _dummy
        self.transaction.savepoint_rollback = _dummy
        self.transaction.enter_transaction_management = _dummy
        self.transaction.leave_transaction_management = _dummy
        if self.django_version > self.DJANGO_1_7:
            self.transaction.atomic = _dummy_context_manager

    def restore_transaction_support(self):
        self.transaction.commit = self.orig_commit
        self.transaction.rollback = self.orig_rollback
        self.transaction.savepoint_commit = self.orig_savepoint_commit
        self.transaction.savepoint_rollback = self.orig_savepoint_rollback
        self.transaction.enter_transaction_management = self.orig_enter
        self.transaction.leave_transaction_management = self.orig_leave
        if self.django_version > self.DJANGO_1_7:
            self.transaction.atomic = self.orig_atomic

    def options(self, parser, env):
        parser.add_option(
            '--django-settings',
            help='Use custom Django settings module.',
            metavar='SETTINGS',
        )
        super(NoseDjango, self).options(parser, env)

    def configure(self, options, conf):
        self.verbosity = conf.verbosity
        if options.django_settings:
            self.settings_module = options.django_settings
        elif 'DJANGO_SETTINGS_MODULE' in os.environ:
            self.settings_module = os.environ['DJANGO_SETTINGS_MODULE']
        else:
            self.settings_module = 'settings'

        super(NoseDjango, self).configure(options, conf)

        self.nose_config = conf

    def call_plugins_method(self, meth_name, *args, **kwargs):
        for plugin in self.django_plugins:
            if hasattr(plugin, meth_name):
                getattr(plugin, meth_name)(*args, **kwargs)

    def begin(self):
        """
        Create the test database and schema, if needed, and switch the
        connection over to that database. Then call install() to install
        all apps listed in the loaded settings module.
        """
        for plugin in self.nose_config.plugins.plugins:
            if getattr(plugin, 'django_plugin', False):
                self.django_plugins.append(plugin)

        os.environ['DJANGO_SETTINGS_MODULE'] = self.settings_module

        if self.conf.addPaths:
            map(add_path, self.conf.where)

        try:
            __import__(self.settings_module)
            self.settings_path = self.settings_module
        except ImportError:
            # Settings module is not found in PYTHONPATH. Try to do
            # some funky backwards crawling in directory tree, ie. add
            # the working directory (and any package parents) to
            # sys.path before trying to import django modules;
            # otherwise, they won't be able to find project.settings
            # if the working dir is project/ or project/..

            self.settings_path = get_settings_path(self.settings_module)

            if not self.settings_path:
                # short circuit if no settings file can be found
                raise RuntimeError("Can't find Django settings file!")

            add_path(self.settings_path)
            sys.path.append(self.settings_path)

        from django.conf import settings

        # Some Django code paths evaluate differently
        # between DEBUG and not DEBUG.  Example of this include the url
        # dispatcher when 404's are hit.  Django's own test runner forces DEBUG
        # to be off.
        settings.DEBUG = False

        self.call_plugins_method('beforeConnectionSetup', settings)

        from django.core import management
        from django.test.utils import setup_test_environment

        if hasattr(settings, 'DATABASES'):
            self.old_db = settings.DATABASES['default']['NAME']
        else:
            self.old_db = settings.DATABASE_NAME
        from django.db import connections

        self._monkeypatch_test_classes()

        for connection in connections.all():
            self.call_plugins_method(
                'beforeTestSetup', settings, setup_test_environment,
                connection)
        setup_test_environment()
        import django
        if hasattr(django, 'setup'):
            django.setup()

        self.call_plugins_method('afterTestSetup', settings)

        management.get_commands()
        # Ensure that nothing (eg. South) steals away our syncdb command
        if self.django_version < self.DJANGO_1_7:
            management._commands['syncdb'] = 'django.core'

        for connection in connections.all():
            self.call_plugins_method(
                'beforeTestDb', settings, connection, management)
            connection.creation.create_test_db(
                verbosity=self.verbosity,
                autoclobber=True,
            )
            logger.debug("Running syncdb")
            self._num_syncdb_calls += 1
            self.call_plugins_method('afterTestDb', settings, connection)
        self.store_original_transaction_methods()

    def _should_use_transaction_isolation(self, test, settings):
        """
        Determine if the given test supports transaction management for
        database rollback test isolation and also whether or not the test has
        opted out of that support.

        Transactions make database rollback much quicker when supported, with
        the caveat that any tests that are explicitly testing transactions
        won't work properly and any tests that depend on external access to the
        test database won't be able to view data created/altered during the
        test.
        """
        if not getattr(test.context, 'use_transaction_isolation', True):
            # The test explicitly says not to use transaction isolation
            return False
        if getattr(settings, 'DISABLE_TRANSACTION_MANAGEMENT', False):
            # Do not use transactions if user has forbidden usage.
            return False
        if hasattr(settings, 'DATABASE_SUPPORTS_TRANSACTIONS'):
            if not settings.DATABASE_SUPPORTS_TRANSACTIONS:
                # The DB doesn't support transactions. Don't try it
                return False

        return True

    def should_use_transaction_isolation(self, test, settings):
        use_transaction_isolation = self._should_use_transaction_isolation(
            test=test,
            settings=settings,
        )
        if self.transaction_is_managed():
            return use_transaction_isolation
        if use_transaction_isolation:
            self.transaction.set_autocommit(False)
        else:
            self.transaction.set_autocommit(True)
        return use_transaction_isolation

    def transaction_is_managed(self):
        if hasattr(self.transaction, 'is_managed'):
            return self.transaction.is_managed()
        return self.transaction.get_connection().in_atomic_block

    def afterTest(self, test):
        """
        Clean up any changes to the test database.
        """
        # Restore transaction support on tests
        from django.conf import settings
        from django.db import connections
        from django.core import mail

        mail.outbox = []

        use_transaction_isolation = self.should_use_transaction_isolation(
            test, settings)

        if use_transaction_isolation:
            self.restore_transaction_support()
            logger.debug("Rolling back")
            self.exit_atomics()
            self.restore_autocommit()
            self.rollback()
            if self.transaction_is_managed():
                self.transaction.leave_transaction_management()
            # If connection is not closed Postgres can go wild with
            # character encodings.
            for connection in connections.all():
                connection.close()
        else:
            # Have to clear the db even if we're using django because django
            # doesn't properly flush the database after a test. It relies on
            # flushing before a test, so we want to avoid the case where a
            # django test doesn't flush and then a normal test runs, because it
            # will expect the db to already be flushed
            self._flush_db()
            self._loaded_test_fixtures = []

        self.call_plugins_method('afterRollback', settings)

    def _flush_db(self):
        from django.core.management import call_command
        call_command('flush', verbosity=0, interactive=False)

        logger.debug("Flushing database")
        self._num_flush_calls += 1

    def beforeTest(self, test):
        """
        Load any database fixtures, set up any test url configurations and
        prepare for using transactions for database rollback if possible.
        """
        if not self.settings_path:
            # short circuit if no settings file can be found
            return

        from django.contrib.sites.models import Site
        from django.contrib.contenttypes.models import ContentType
        from django.core.management import call_command
        from django.core.urlresolvers import clear_url_caches
        from django.conf import settings

        use_transaction_isolation = self.should_use_transaction_isolation(
            test, settings)

        if use_transaction_isolation:
            self.call_plugins_method(
                'beforeTransactionManagement',
                settings,
                test,
            )
            self.transaction.enter_transaction_management()
            self.transaction.managed(True)
            self.restore_transaction_support()
            self.enable_autocommit()
            self.disable_transaction_support()

        Site.objects.clear_cache()
        # Otherwise django.contrib.auth.Permissions will depend on deleted
        # ContentTypes
        ContentType.objects.clear_cache()

        if use_transaction_isolation:
            self.call_plugins_method(
                'afterTransactionManagement',
                settings,
                test,
            )

        self.call_plugins_method('beforeFixtureLoad', settings, test)
        if isinstance(test, nose.case.Test):
            # Mirrors django.test.testcases:TestCase

            fixtures_to_load = getattr(test.context, 'fixtures', [])
            if fixtures_to_load is None:
                fixtures_to_load = []
            # We have to use this slightly awkward syntax due to the fact
            # that we're using *args and **kwargs together.
            ordered_fixtures = sorted(fixtures_to_load)
            if ordered_fixtures != self._loaded_test_fixtures:
                # Only clear + load the fixtures if they're not already
                # loaded

                # Flush previous fixtures
                if use_transaction_isolation:
                    self.restore_transaction_support()

                self._flush_db()

                # Load the new fixtures
                logger.debug("Loading fixtures: %s", fixtures_to_load)
                if fixtures_to_load:
                    commit = True
                    if use_transaction_isolation:
                        commit = False
                    call_command(
                        'loaddata',
                        *test.context.fixtures,
                        **{'verbosity': 0, 'commit': commit}
                    )
                if use_transaction_isolation:
                    self.restore_transaction_support()
                    self.commit()
                    self.disable_transaction_support()
                self._num_fixture_loads += 1
                self._loaded_test_fixtures = ordered_fixtures
        self.call_plugins_method('afterFixtureLoad', settings, test)

        self.call_plugins_method('beforeUrlConfLoad', settings, test)
        if isinstance(test, nose.case.Test) and \
           hasattr(test.context, 'urls'):
            # We have to use this slightly awkward syntax due to the fact
            # that we're using *args and **kwargs together.
            self.old_urlconf = settings.ROOT_URLCONF
            settings.ROOT_URLCONF = test.context.urls
            clear_url_caches()
        self.call_plugins_method('afterUrlConfLoad', settings, test)
        if self.django_version > self.DJANGO_1_7:
            self.disable_transaction_support()
        if use_transaction_isolation:
            self.restore_transaction_support()
            self.enter_atomics()
            self.disable_transaction_support()
        if hasattr(test.context, 'client_class'):
            test.context.client = test.context.client_class()

    def finalize(self, result=None):
        """
        Clean up any created database and schema.
        """
        if not self.settings_path:
            # short circuit if no settings file can be found
            return

        from django.test.utils import teardown_test_environment
        from django.db import connection
        from django.conf import settings
        from django.core.urlresolvers import clear_url_caches

        self.call_plugins_method('beforeDestroyTestDb', settings, connection)
        try:
            connection.creation.destroy_test_db(
                self.old_db,
                verbosity=self.verbosity,
            )
        except Exception:
            # If we can't tear down the test DB, don't worry about it.
            pass
        self.call_plugins_method('afterDestroyTestDb', settings, connection)

        self.call_plugins_method(
            'beforeTeardownTestEnv', settings, teardown_test_environment)
        teardown_test_environment()
        self.call_plugins_method('afterTeardownTestEnv', settings)

        if hasattr(self, 'old_urlconf'):
            settings.ROOT_URLCONF = self.old_urlconf
            clear_url_caches()

    def report(self, stream):
        stream.writeln("Loaded fixtures %s times" % self._num_fixture_loads)
        stream.writeln("Flushed the db %s times" % self._num_flush_calls)
        stream.writeln("Sync'd the db %s times" % self._num_syncdb_calls)

    def _monkeypatch_test_classes(self):
        # Monkeypatching. Like a boss.
        # We're taking over all of the fixture management and such from the
        # django test classes. Dealing with switching back and forth between
        # them is a huge pita, and this just kind of works.
        import django.test.testcases

        overrides = [
            '_pre_setup',
            '_fixture_setup',
            '_urlconf_setup',
            '_post_teardown',
            '_fixture_teardown',
            '_urlconf_teardown',
        ]
        override_class_methods = [
            'setUpClass',
            'tearDownClass',
        ]

        @classmethod
        def _dummy_class_method(*args, **kwargs):
            pass

        for override in overrides:
            setattr(
                django.test.testcases.TransactionTestCase,
                override,
                _dummy,
            )
            setattr(django.test.testcases.TestCase, override, _dummy)

        for override in override_class_methods:
            setattr(
                django.test.testcases.TransactionTestCase,
                override,
                _dummy_class_method,
            )
            setattr(
                django.test.testcases.TestCase,
                override,
                _dummy_class_method,
            )

        setattr(
            django.test.testcases.TransactionTestCase,
            'use_transaction_isolation',
            False,
        )
        setattr(
            django.test.testcases.TestCase,
            'use_transaction_isolation',
            True,
        )



from django.test import TestCase
from project.zoo.models import Zoo

class TestDjango(TestCase):
    def testcase1(self):
        zoo = Zoo.objects.create(name='blah')
        assert Zoo.objects.count() == 1

    def testcase2(self):
        zoo = Zoo.objects.create(name='blah')
        assert Zoo.objects.count() == 1




from django.test import TestCase
from django.test.client import Client

class TestStandardUrlConf(TestCase):
    def test_index(self):
        '''
        We're using the standard ROOT_URLCONF, so we need to
        pass in /zoo/, just the empty string
        '''
        c = Client()
        resp = c.get('')
        assert resp.status_code == 404, "Got status %s - expecting 404" % resp.status_code

        c = Client()
        resp = c.get('/zoo/')
        assert "Just a title" in resp.content
        assert "foobar" in resp.content

class TestCustomUrlConf(TestCase):
    urls = 'zoo.urls'

    def test_index(self):
        '''
        We're customizing the ROOT_URLCONF with zoo.urls,
        so we do *not* need to pass in /zoo/, just the empty string
        '''
        c = Client()
        resp = c.get('')
        assert "Just a title" in resp.content
        assert "foobar" in resp.content

        c = Client()
        resp = c.get('/zoo/')
        assert resp.status_code == 404, "Got status %s - expecting 404" % resp.status_code







import os
import time

from django.core.handlers.wsgi import WSGIHandler
from nosedjango.plugins.base_plugin import Plugin

# Next 3 plugins taken from django-sane-testing:
# http://github.com/Almad/django-sane-testing
# By: Lukas "Almad" Linhart http://almad.net/
#####
# It was a nice try with Django server being threaded.
# It still sucks for some cases (did I mentioned urllib2?),
# so provide cherrypy as working alternative.
# Do imports in method to avoid CP as dependency
# Code originally written by Mikeal Rogers under Apache License.
#####

DEFAULT_LIVE_SERVER_ADDRESS = '0.0.0.0'
DEFAULT_LIVE_SERVER_PORT = '8000'


class CherryPyLiveServerPlugin(Plugin):
    name = 'cherrypyliveserver'
    activation_parameter = '--with-cherrypyliveserver'
    nosedjango = True

    def __init__(self):
        Plugin.__init__(self)
        self.server_started = False
        self.server_thread = None

    def options(self, parser, env=os.environ):
        Plugin.options(self, parser, env)

    def configure(self, options, config):
        Plugin.configure(self, options, config)

    def startTest(self, test):
        from django.conf import settings

        if not self.server_started and \
           getattr(test, 'start_live_server', False):

            self.start_server(
                address=getattr(
                    settings,
                    "LIVE_SERVER_ADDRESS",
                    DEFAULT_LIVE_SERVER_ADDRESS,
                ),
                port=int(getattr(
                    settings,
                    "LIVE_SERVER_PORT",
                    DEFAULT_LIVE_SERVER_PORT,
                ))
            )
            self.server_started = True

    def finalize(self, result):
        self.stop_test_server()

    def start_server(self, address='0.0.0.0', port=8000):
        from django.contrib.staticfiles.handlers import StaticFilesHandler
        _application = StaticFilesHandler(WSGIHandler())

        def application(environ, start_response):
            environ['PATH_INFO'] = environ['SCRIPT_NAME'] + environ['PATH_INFO']  # noqa
            return _application(environ, start_response)

        from cherrypy.wsgiserver import CherryPyWSGIServer
        from threading import Thread
        self.httpd = CherryPyWSGIServer(
            (address, port),
            application,
            server_name='django-test-http',
        )
        self.httpd_thread = Thread(target=self.httpd.start)
        self.httpd_thread.start()
        # FIXME: This could be avoided by passing self to thread class starting
        # django and waiting for Event lock
        time.sleep(.5)

    def stop_test_server(self):
        if self.server_started:
            self.httpd.stop()
            self.server_started = False



import os.path

from nosedjango.plugins.base_plugin import Plugin

CONFIG_FILES = [
    # Linux users will prefer this
    "~/.djangosettingstargetrc",
    # Windows users will prefer this
    "~/.djangosettingstarget.cfg",
]


class SettingsTargetPlugin(Plugin):
    """
    Switch Django settings values based on a settings ``target`` input. Allows
    for the creation of different groups of settings that can be triggered
    based on different target values.

    For example, you might optionally use Amazon S3 as a file storage backend.
    Most of the time, you wouldn't want to actually use S3 when running unit
    tests, but sometimes you do want to actually test that integration. You
    would set up one target with actual S3 backend settings while using the
    DEFAULT target to blank them out.
    """
    name = 'django-settings-target'

    def options(self, parser, env):
        parser.add_option(
            '--django-settings-target-conf',
            help='Path to your settings target configuration file',
            default=None,
        )
        parser.add_option(
            '--django-settings-target',
            help='Named settings group or "target" to use for testing.',
            default=None,
        )
        super(SettingsTargetPlugin, self).options(parser, env)

    def get_config_files(self):
        return filter(os.path.exists,
                      map(os.path.expanduser, CONFIG_FILES))



from nosedjango.plugins.base_plugin import Plugin


class SqlitePlugin(Plugin):
    """
    Modify django database settings to use an in-memory sqlite instance for
    faster test runs and easy multiprocess testing.
    """
    name = 'django-sqlite'

    def beforeConnectionSetup(self, settings):
        settings.DATABASES['default']['ENGINE'] = 'django.db.backends.sqlite3'  # noqa
        settings.DATABASES['default']['NAME'] = ':memory:'
        settings.DATABASES['default']['OPTIONS'] = {}
        settings.DATABASES['default']['USER'] = ''
        settings.DATABASES['default']['PASSWORD'] = ''



import math
import random
import string

from nose.plugins.base import Plugin as NosePlugin


class Plugin(NosePlugin):
    django_plugin = True
    _unique_token = None

    def get_unique_token(self):
        """
        Get a unique token for usage in differentiating test runs that need to
        run in parallel.
        """
        if self._unique_token is None:
            self._unique_token = self._random_token()

        return self._unique_token

    def _random_token(self, bits=128):
        """
        Generates a random token, using the url-safe base64 alphabet.
        The "bits" argument specifies the bits of randomness to use.
        """
        alphabet = string.ascii_letters + string.digits + '-_'
        # alphabet length is 64, so each letter provides lg(64) = 6 bits
        num_letters = int(math.ceil(bits / 6.0))
        return ''.join(random.choice(alphabet) for i in range(num_letters))


class IPluginInterface(object):
    """
    IPluginInteface describes the NoseDjango plugin API. Do not subclass or use
    this class directly. This interface describes an API **in addition** to
    the API described with ``nose.plugins.base.IPluginInterface``.
    """
    def __new__(cls, *arg, **kw):
        raise TypeError("IPluginInterface class is for documentation only")

    def beforeConnectionSetup(self, settings):
        pass

    def beforeTestSetup(self, settings, setup_test_environment, connection):
        pass

    def afterTestSetup(self, settings):
        pass

    def beforeTestDb(self, settings, connection, management):
        pass

    def afterTestDb(self, settings, connection):
        pass

    def beforeTransactionManagement(self, settings, test):
        pass

    def afterTransactionManagement(self, settings, test):
        pass

    def beforeFixtureLoad(self, settings, test):
        pass

    def afterFixtureLoad(self, settings, test):
        pass

    def beforeUrlConfLoad(self, settings, test):
        pass

    def afterUrlConfLoad(self, settings, test):
        pass

    def beforeDestroyTestDb(self, settings, connection):
        pass

    def afterDestroyTestDb(self, settings, connection):
        pass

    def beforeTeardownTestEnv(self, settings, teardown_test_environment):
        pass

    def afterTeardownTestEnv(self, settings):
        pass



import os
import subprocess
import sys
import signal

from nosedjango.plugins.base_plugin import Plugin


class SshTunnelPlugin(Plugin):
    name = 'sshtunnel'

    def options(self, parser, env=None):
        if env is None:
            env = os.environ
        parser.add_option(
            '--remote-server',
            help='Use a remote server to run the tests, must pass in the server address',  # noqa
        )
        parser.add_option(
            '--to-from-ports',
            help=(
                'Should be of the form x:y where x is the port that needs to '
                'be forwarded to the server and y is the port that the server '
                'needs forwarded back to the localhost'
            ),
            default='4444:8001',
        )
        parser.add_option(
            '--username',
            help='The username with which to create the ssh tunnel to the remote server',  # noqa
            default=None,
        )
        Plugin.options(self, parser, env)

    def configure(self, options, config):
        # This is only checked since this plugin is configured regardless if
        # the sshtunnel flag is used, and we only want this info here if the
        # --remote-server flag is used
        if options.remote_server:
            try:
                to_port, from_port = options.to_from_ports.split(':', 1)
            except:
                raise RuntimeError("--to_from_ports should be of the form x:y")
            else:
                self._to_port = to_port
                self._from_port = from_port
            self._remote_server = options.remote_server
            self._username = options.username
        Plugin.configure(self, options, config)

    def begin(self):
        # If we are using a remote connection we want to create two tunnels,
        # one to forward from local to server for the port that selenium is
        # listening to, and another from server to local on the port runserver
        # is running on
        if getattr(self, '_remote_server', False):
            params = {
                'username': self._username,
                'host': self._remote_server,
                'to_port': self._to_port,
                'from_port': self._from_port,
            }
            host_str = self._remote_server
            if self._username:
                host_str = '%(username)s@%(host)s' % params,
            self.tunnel_command = [
                'ssh',
                host_str,
                '-L',
                '%(to_port)s:%(host)s:%(to_port)s' % params,
                '-N',
            ]
            self.reverse_tunnel_command = [
                'ssh',
                '-nNT',
                '-R',
                '%(from_port)s:localhost:%(from_port)s' % params,
                host_str,
            ]
            self._tunnel = subprocess.Popen(
                self.tunnel_command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            self._reverse_tunnel = subprocess.Popen(
                self.reverse_tunnel_command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )

    def finalize(self, result):
        # Clean up all ssh tunnels
        if not self._tunnel.poll():
            os.kill(self._tunnel.pid, signal.SIGKILL)
            self._tunnel.wait()
        if not self._reverse_tunnel.poll():
            os.kill(self._reverse_tunnel.pid, signal.SIGKILL)
            self._reverse_tunnel.wait()

            print sys.stderr, "Error killing ssh tunnel"




def test_regular_test_method():
    ''' This is just a stub for a regular test method'''
    pass

def test_docstring():
    '''
    This is a doctest that might get skipped

    >>> 5 + 5 
    10
    '''




import os
import logging
import subprocess
import urllib2
import httplib
import time
from pprint import pprint

import nose.case

from nosedjango.plugins.base_plugin import Plugin


class SeleniumPlugin(Plugin):
    name = 'selenium'

    def options(self, parser, env=None):
        if env is None:
            env = os.environ
        parser.add_option(
            '--selenium-ss-dir',
            help='Directory for failure screen shots.'
        )
        parser.add_option(
            '--headless',
            help=(
                "Run the Selenium tests in a headless mode, with virtual "
                "frames starting with the given index (eg. 1)"
            ),
            default=None,
        )
        parser.add_option(
            '--driver-type',
            help='The type of driver that needs to be created',
            default='firefox',
        )
        parser.add_option(
            '--firefox-binary',
            help='the path to the firefox browser binary',
            default=None,
        )
        parser.add_option(
            '--remote-server-address',
            help='Use a remote server to run the tests, must pass in the server address',  # noqa
            default='localhost',
        )
        parser.add_option(
            '--selenium-port',
            help='The port for the selenium server',
            default='4444',
        )
        parser.add_option(
            '--track-stats',
            help=(
                'After the suite is run print a table of test '
                'name/runtime/number of trips to the server.  defaults to '
                'ordering by trips to the server'
            ),
            default=None,
        )
        Plugin.options(self, parser, env)

    def configure(self, options, config):
        if options.selenium_ss_dir:
            self.ss_dir = os.path.abspath(options.selenium_ss_dir)
        else:
            self.ss_dir = os.path.abspath('failure_screenshots')
        valid_browsers = ['firefox', 'internet_explorer', 'chrome']
        if options.driver_type not in valid_browsers:
            raise RuntimeError(
                '--driver-type must be one of: %s' % ' '.join(valid_browsers)
            )
        self._firefox_binary = options.firefox_binary
        self._driver_type = options.driver_type.replace('_', ' ')
        self._remote_server_address = options.remote_server_address
        self._selenium_port = options.selenium_port
        self._driver = None
        self._current_windows_handle = None

        self.x_display = 1
        self.run_headless = False
        if options.headless:
            self.run_headless = True
            self.x_display = int(options.headless)
        if options.track_stats and options.track_stats not in ('trips', 'runtime'):  # noqa
            raise RuntimeError('--track-stats must be "trips" or "runtime"')
        self._track_stats = options.track_stats
        Plugin.configure(self, options, config)

    def get_driver(self):
        from selenium.webdriver import Firefox as FirefoxWebDriver
        from selenium.webdriver import Chrome as ChromeDriver
        from selenium.webdriver import Remote as RemoteDriver

        # Lazilly gets the driver one time cant call in begin since ssh tunnel
        # may not be created
        if self._driver:
            return self._driver

        if self._driver_type == 'firefox':
            if self._firefox_binary is None:
                self._driver = FirefoxWebDriver()
            else:
                from selenium.webdriver.firefox.firefox_binary import FirefoxBinary  # noqa
                binary = FirefoxBinary(self._firefox_binary)
                self._driver = FirefoxWebDriver(firefox_binary=binary)
        elif self._driver_type == 'chrome':
            self._driver = ChromeDriver()
        else:
            timeout = 60
            step = 1
            current = 0
            while current < timeout:
                try:
                    self._driver = RemoteDriver(
                        'http://%s:%s/wd/hub' % (
                            self._remote_server_address,
                            self._selenium_port,
                        ),
                        self._driver_type,
                        'WINDOWS',
                    )
                    break
                except urllib2.URLError:
                    time.sleep(step)
                    current += step
                except httplib.BadStatusLine:
                    self._driver = None
                    break
            if current >= timeout:
                raise urllib2.URLError('timeout')

        monkey_patch_methods(self._driver)
        return self._driver

    def finalize(self, result):
        if self._track_stats and self.times:
            print '-' * 80
            if self._track_stats == 'runtime':
                order = 1
            elif self._track_stats == 'trips':
                order = 2
            pprint(sorted(self.times, key=lambda x: x[order]))
            print '-' * 80
        driver = self.get_driver()
        if driver:
            driver.quit()

        if self.xvfb_process:
            os.kill(self.xvfb_process.pid, 9)
            os.waitpid(self.xvfb_process.pid, 0)

    def begin(self):
        self.xvfb_process = None
        if self.run_headless:
            xvfb_display = self.x_display
            try:
                self.xvfb_process = subprocess.Popen(
                    [
                        'xvfb',
                        ':%s' % xvfb_display,
                        '-ac',
                        '-screen',
                        '0',
                        '1024x768x24',
                    ],
                    stderr=subprocess.PIPE,
                )
            except OSError:
                # Newer distros use Xvfb
                self.xvfb_process = subprocess.Popen(
                    [
                        'Xvfb',
                        ':%s' % xvfb_display,
                        '-ac',
                        '-screen',
                        '0',
                        '1024x768x24',
                    ],
                    stderr=subprocess.PIPE,
                )
            os.environ['DISPLAY'] = ':%s' % xvfb_display

    def beforeTest(self, test):
        self.start_time = time.time()
        driver = self.get_driver()
        logging.getLogger().setLevel(logging.INFO)
        setattr(test.test, 'driver', driver)
        # need to know the main window handle for cleaning up extra windows at
        # the end of each test
        if driver:
            self._current_windows_handle = driver.current_window_handle

    def afterTest(self, test):
        if not hasattr(self, 'times'):
            self.times = []
        if self.times and self._driver:
            self.times.append((
                test.address()[2],
                int(time.time() - self.start_time),
                self._driver.roundtrip_counter
            ))
            self._driver.roundtrip_counter = 0
        driver = getattr(test.test, 'driver', False)
        if not driver:
            return
        if self._current_windows_handle:
            # close all extra windows except for the main window
            for window in driver.window_handles:
                if window != self._current_windows_handle:
                    driver.switch_to_window(window)
                    driver.close()
                    driver.switch_to_window(self._current_windows_handle)

    def handleError(self, test, err):
        if isinstance(test, nose.case.Test) and \
           getattr(test.context, 'selenium_take_ss', False):
            self._take_screenshot(test)

    def handleFailure(self, test, err):
        if isinstance(test, nose.case.Test) and \
           getattr(test.context, 'selenium_take_ss', False):
            self._take_screenshot(test)

    def _take_screenshot(self, test):
        driver_attr = getattr(test.context, 'selenium_driver_attr', 'driver')
        try:
            driver = getattr(test.test, driver_attr)
        except AttributeError:
            print "Error attempting to take failure screenshot"
            return

        # Make the failure ss directory if it doesn't exist
        if not os.path.exists(self.ss_dir):
            os.makedirs(self.ss_dir)

        ss_file = os.path.join(self.ss_dir, '%s.png' % test.id())

        # The Remote server does not have the attribute ``save_screenshot``, so
        # we have to check to see if it is there before using it
        if hasattr(driver, 'save_screenshot'):
            driver.save_screenshot(ss_file)


def monkey_patch_methods(driver):
    # Keep track of how many trips to execute are made
    old_execute = driver.__class__.execute

    def new_execute(self, *args, **kwargs):
        roundtrip_counter = getattr(driver, 'roundtrip_counter', 0)
        driver.roundtrip_counter = roundtrip_counter + 1
        return old_execute(self, *args, **kwargs)
    driver.__class__.execute = new_execute

    # If there is an alert when trying to get a page, accept it
    old_get = driver.__class__.get

    def new_get(self, *args, **kwargs):
        old_get(self, *args, **kwargs)
        accept_alert(driver)
    driver.__class__.get = new_get

    # Need to move away from the page to ensure if there is an alert on the
    # page it gets dealt with prior to closing the window
    old_close = driver.__class__.close

    def new_close(self, *args, **kwargs):
        # page to ensure page is changed
        self.get('http://www.google.com')
        old_close(self, *args, **kwargs)
    driver.__class__.close = new_close

    old_quit = driver.__class__.quit

    def new_quit(self, *args, **kwargs):
        self.close()  # Need to handle onbeforeunload alerts
        old_quit(self, *args, **kwargs)
    driver.__class__.quit = new_quit


def accept_alert(driver):
    from selenium.common.exceptions import WebDriverException

    alert = driver.switch_to_alert()
    try:
        alert.accept()
    except WebDriverException:
        pass



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Do nothing, other than shadow django's syncdb command."

    def handle(self, *args, **options):
        pass



from django.test import TestCase
from django.test.client import Client


class NoseClient(Client):
    is_nose = True

    def __init__(self):
        self.items = []


class CustomClientTestCase(TestCase):
    client_class = NoseClient

    def test_client_is_nose(self):
        assert self.client.is_nose

    def test_AAA_change_client_items(self):
        self.client.items.append(1)

    def test_BBB_client_items_empty(self):
        self.assertEqual(self.client.items, [])



import datetime
from unittest import TestCase as UnitTestCase

from nose.plugins.skip import SkipTest

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from nosedjangotests.polls.models import Poll, Choice


def _test_using_content_types(self):
    p1, _ = Poll.objects.get_or_create(
        question='Who you?', pub_date=datetime.datetime.now())

    choice1 = Choice(poll=p1, choice='me')
    choice1.save()


def _test_get_contenttypes(self):
    models = [Poll, Choice]

    for model in models:
        content_type = ContentType.objects.get_for_model(model)

        # Make sure this isn't just using the cache
        ct_db = ContentType.objects.get(pk=content_type.pk)
        for attr in ['name', 'app_label', 'model']:
            self.assertEqual(
                getattr(ct_db, attr), getattr(content_type, attr))


def _test_permissions(self):
    perm_types = ['add', 'change', 'delete']
    models = [(Poll, 'poll'), (Choice, 'choice')]

    for model, name in models:
        for perm_type in perm_types:
            codename = '%s_%s' % (perm_type, name)
            content_type = ContentType.objects.get_for_model(model)
            num_perms = Permission.objects.filter(
                codename=codename, content_type=content_type).count()
            self.assertEqual(num_perms, 1)


def _test_fixtures_1(self):
    num_polls = Poll.objects.all().count()
    self.assertEqual(num_polls, 1)

    bear_poll = Poll.objects.get(pk=1)
    self.assertEqual(bear_poll.question, 'What bear is best?')

    new_poll = Poll.objects.create(
        question="Did my shoes come off in the plane crash?",
        pub_date=datetime.datetime.now())
    new_poll.save()


def _test_fixtures_2(self):
    num_polls = Poll.objects.all().count()
    self.assertEqual(num_polls, 2)

    fox_poll = Poll.objects.get(pk=1)
    self.assertEqual(fox_poll.question, 'What does the fox say?')

    wood_poll = Poll.objects.get(pk=2)
    self.assertEqual(wood_poll.question, 'Is there fire wood on the island?')

    new_poll = Poll.objects.create(
        question="Did my shoes come off in the plane crash?",
        pub_date=datetime.datetime.now())
    new_poll.save()


class DjangoTestCase(TestCase):
    fixtures = ['polls1.json']

    def test_a_skip(self):
        raise SkipTest('Skipping')

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_get_contenttypes_1(self):
        _test_get_contenttypes(self)

    def test_permissions_1(self):
        _test_permissions(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)

    def test_get_contenttypes_2(self):
        _test_get_contenttypes(self)

    def test_permissions_2(self):
        _test_permissions(self)

    def test_fixtures_1(self):
        _test_fixtures_1(self)

    def test_fixtures_2(self):
        _test_fixtures_1(self)


class DjangoTestCase2(DjangoTestCase):
    fixtures = ['polls2.json']

    def test_fixtures_1(self):
        _test_fixtures_2(self)

    def test_fixtures_2(self):
        _test_fixtures_2(self)


class DjangoTransactionTestCase(TestCase):
    fixtures = ['polls1.json']

    def test_a_skip(self):
        raise SkipTest('Skipping')

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_get_contenttypes_1(self):
        _test_get_contenttypes(self)

    def test_permissions_1(self):
        _test_permissions(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)

    def test_get_contenttypes_2(self):
        _test_get_contenttypes(self)

    def test_permissions_2(self):
        _test_permissions(self)

    def test_fixtures_1(self):
        _test_fixtures_1(self)

    def test_fixtures_2(self):
        _test_fixtures_1(self)


class DjangoTransactionTestCase2(DjangoTransactionTestCase):
    fixtures = ['polls2.json']

    def test_fixtures_1(self):
        _test_fixtures_2(self)

    def test_fixtures_2(self):
        _test_fixtures_2(self)


class WithTransactionUnitTestCase(UnitTestCase):
    fixtures = ['polls1.json']

    def test_a_skip(self):
        raise SkipTest('Skipping')

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_get_contenttypes_1(self):
        _test_get_contenttypes(self)

    def test_permissions_1(self):
        _test_permissions(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)

    def test_get_contenttypes_2(self):
        _test_get_contenttypes(self)

    def test_permissions_2(self):
        _test_permissions(self)

    def test_fixtures_1(self):
        _test_fixtures_1(self)

    def test_fixtures_2(self):
        _test_fixtures_1(self)


class WithTransactionUnitTestCase2(WithTransactionUnitTestCase):
    fixtures = ['polls2.json']

    def test_fixtures_1(self):
        _test_fixtures_2(self)

    def test_fixtures_2(self):
        _test_fixtures_2(self)


class NoTransactionUnitTestCase(UnitTestCase):
    fixtures = ['polls1.json']
    use_transaction_isolation = False

    def test_a_skip(self):
        raise SkipTest('Skipping')

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_get_contenttypes_1(self):
        _test_get_contenttypes(self)

    def test_permissions_1(self):
        _test_permissions(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)

    def test_get_contenttypes_2(self):
        _test_get_contenttypes(self)

    def test_permissions_2(self):
        _test_permissions(self)

    def test_fixtures_1(self):
        _test_fixtures_1(self)

    def test_fixtures_2(self):
        _test_fixtures_1(self)


class NoTransactionUnitTestCase2(NoTransactionUnitTestCase):
    fixtures = ['polls2.json']

    def test_fixtures_1(self):
        _test_fixtures_2(self)

    def test_fixtures_2(self):
        _test_fixtures_2(self)


class MultipleFixtureTestCase(UnitTestCase):
    fixtures = ['polls1.json', 'polls2.json']

    def test_a_skip(self):
        raise SkipTest('Skipping')

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_get_contenttypes_1(self):
        _test_get_contenttypes(self)

    def test_permissions_1(self):
        _test_permissions(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)

    def test_get_contenttypes_2(self):
        _test_get_contenttypes(self)

    def test_permissions_2(self):
        _test_permissions(self)



from django.db import models
try:
    from django.db.transaction import atomic
except ImportError:  # Django 1.4
    from django.db.transaction import commit_on_success as atomic
from django.contrib.contenttypes import generic


class Poll(models.Model):
    question = models.CharField(max_length=200)
    pub_date = models.DateTimeField('date published')

    @atomic
    def reset(self):
        self.question = ""
        self.save()


class Choice(models.Model):
    content_type = models.ForeignKey('contenttypes.ContentType')
    object_id = models.PositiveIntegerField()
    poll = generic.GenericForeignKey('content_type', 'object_id')

    choice = models.CharField(max_length=200)
    votes = models.IntegerField(default=0)

    @atomic
    def reset(self):
        self.votes = 0
        self.save()



from django.test import TransactionTestCase


class SeleniumTestCase(TransactionTestCase):
    def test_modified_binary_path_is_passed_into_driver(self):
        driver = self.driver

        self.assertIn('////', driver.binary._start_cmd)



import codecs
import os
import sys

from setuptools import setup, find_packages, Command


class RunTestBase(Command):
    description = "Run the test suite from the tests dir."
    user_options = []
    extra_env = {}

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass


class RunTests(RunTestBase):
    test_app = 'nosedjangotests.polls'
    check_selenium = False
    label = ''
    args = []

    def verify_selenium(self):
        if not self.check_selenium:
            return
        try:
            import selenium  # noqa
        except ImportError:
            print "Selenium not installed. Skipping tests."
            sys.exit(0)

    def run(self):
        for env_name, env_value in self.extra_env.items():
            os.environ[env_name] = str(env_value)

        setup_dir = os.path.abspath(os.path.dirname(__file__))
        tests_dir = os.path.join(setup_dir, 'nosedjangotests')
        os.chdir(tests_dir)
        sys.path.append(tests_dir)

        try:
            from nose.core import TestProgram
            import nosedjango
            print nosedjango.__version__
        except ImportError:
            print 'nose and nosedjango are required to run this test suite'
            sys.exit(1)

        print "Running tests with {label}".format(label=self.label)
        self.verify_selenium()

        test_args = [
            '-v',
            '--verbosity=2',
            '--with-doctest',
            '--with-django',
            '--django-settings', 'nosedjangotests.settings',
            self.test_app,
        ] + self.args
        TestProgram(argv=test_args, exit=True)


class SQLiteTestCase(RunTests):
    label = 'sqlite'
    args = [
        '--with-django-sqlite',
    ]


class MultiProcessTestCase(RunTests):
    label = 'sqlite and multiprocess'
    args = [
        '--with-django-sqlite',
        '--processes', '3',
    ]


class MySQLTestCase(RunTests):
    label = 'MySQL'


class CherryPyLiveServerTestCase(RunTests):
    label = 'Cherry Py Test Case'
    args = [
        '--with-django-sqlite',
        '--with-cherrypyliveserver',
    ]


class SeleniumTestCase(RunTests):
    label = 'Selenium'
    test_app = 'nosedjangotests.selenium_tests'
    check_selenium = True
    args = [
        '--with-selenium',
    ]

class SeleniumBinaryTestCase(RunTests):
    label = 'Selenium Binary Option'
    test_app = 'nosedjangotests.selenium_binary_tests'
    check_selenium = True
    args = [
        '--with-django-sqlite',
        '--with-selenium',
    ]

    def run(self):
        self.verify_selenium()
        from selenium.webdriver.firefox.firefox_binary import FirefoxBinary

        # borrow selenium's magical binary finding logic
        dummy_binary = FirefoxBinary()
        ff_path = dummy_binary._start_cmd.strip()

        # mangle the path with a no-op of prefix slashes.
        # symlinks run into very nasty issues with newer versions of Firefox on OSX

        self.args.append('--firefox-binary=////%s' % ff_path)

        RunTests.run(self)

import nosedjango

long_description = codecs.open("README.rst", "r", "utf-8").read()

setup(
    name='nosedjango',
    version=nosedjango.__version__,
    description=nosedjango.__doc__,
    author=nosedjango.__author__,
    author_email=nosedjango.__contact__,
    long_description=long_description,
    install_requires=['nose<2.0', 'django'],
    extras_require={
        'selenium': ['selenium>=2.0'],
    },
    dependency_links=['http://bitbucket.org/jpellerin/nose/get/release_0.11.4.zip#egg=nose-0.11.4.dev'],  # noqa
    url="http://github.com/nosedjango/nosedjango",
    license='GNU LGPL',
    packages=find_packages(exclude=['nosedjangotests', 'nosedjangotests.*']),
    zip_safe=False,
    cmdclass={
        'test_sqlite': SQLiteTestCase,
        'test_multiprocess': MultiProcessTestCase,
        'test_mysql': MySQLTestCase,
        'test_selenium': SeleniumTestCase,
        'test_selenium_binary': SeleniumBinaryTestCase,
        'test_cherrypy': CherryPyLiveServerTestCase,
    },
    include_package_data=True,
    entry_points={
        'nose.plugins': [
            'celery = nosedjango.plugins.celery_plugin:CeleryPlugin',
            'cherrypyliveserver = nosedjango.plugins.cherrypy_plugin:CherryPyLiveServerPlugin',  # noqa
            'django = nosedjango.nosedjango:NoseDjango',
            'djangofilestorage = nosedjango.plugins.file_storage_plugin:FileStoragePlugin',  # noqa
            'djangosphinxsearch = nosedjango.plugins.sphinxsearch_plugin:SphinxSearchPlugin',  # noqa
            'djangosqlite = nosedjango.plugins.sqlite_plugin:SqlitePlugin',
            'selenium = nosedjango.plugins.selenium_plugin:SeleniumPlugin',
            'sshtunnel = nosedjango.plugins.ssh_tunnel_plugin:SshTunnelPlugin',
        ],
    },
)



from django.test import TestCase
from django.core import mail


class MailBoxTestCase(TestCase):
    def test_AAA_send_mail(self):
        self.assertEqual(len(mail.outbox), 0)
        mail.send_mail(
            subject='subject',
            message='body',
            from_email='foo@gmail.com',
            recipient_list=['bar@gmail.com'],
        )
        self.assertEqual(len(mail.outbox), 1)

    def test_BBB_send_mail(self):
        self.test_AAA_send_mail()



from contextlib import contextmanager
from datetime import datetime

import django
try:
    from django.db.transaction import atomic
except ImportError:
    from django.db.transaction import commit_on_success as atomic
from django.db import transaction
from django.db.transaction import TransactionManagementError
from django.test import TestCase

from nose.plugins.skip import SkipTest

from nosedjangotests.polls.models import Choice, Poll


def _test_using_content_types(self):
    self.assertEqual(Choice.objects.all().count(), 1)


class ContentTypeTestCase(TestCase):
    fixtures = [
        'polls1.json',
        'choices.json',
    ]

    def test_using_content_types_1(self):
        _test_using_content_types(self)

    def test_using_content_types_2(self):
        _test_using_content_types(self)


class CherryPyTestCase(ContentTypeTestCase):
    start_live_server = True


@contextmanager
def set_autocommit(value):
    old_value = transaction.get_autocommit()
    transaction.set_autocommit(value)
    yield
    transaction.set_autocommit(old_value)


class FixtureBleedThroughTestCase(TestCase):
    use_transaction_isolation = True

    fixtures = [
        'polls1.json',
        'choices.json',
    ]

    def test_AAA_change_poll_question(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')
        poll = Poll.objects.get()
        poll.question = 'What does the fox say?'

        # If we are using transaction isolation, then using atomic/autocommit
        # is invalid.
        if self.use_transaction_isolation:
            with self.assertRaises(TransactionManagementError):
                with set_autocommit(True):
                    with atomic():
                        poll.save()
        else:
            with set_autocommit(True):
                with atomic():
                    poll.save()

    def test_BBB_poll_question_is_what_is_in_the_fixture(self):
        poll = Poll.objects.get()
        self.assertEqual(poll.question, 'What bear is best?')


class NonFixtureBleedThroughTestCase(TestCase):
    use_transaction_isolation = True

    def test_AAA_create_a_poll(self):
        if django.VERSION < (1, 7):
            raise SkipTest('This test is only an issue in django 1.8+')
        # If we are using transaction isolation, then using atomic/autocommit
        # is invalid.
        if self.use_transaction_isolation:
            with self.assertRaises(TransactionManagementError):
                with set_autocommit(True):
                    with atomic():
                        Poll.objects.create(
                            question='Test',
                            pub_date=datetime.now(),
                        )
        else:
            with set_autocommit(True):
                with atomic():
                    Poll.objects.create(
                        question='Test',
                        pub_date=datetime.now(),
                    )

    def test_BBB_no_polls_in_the_db(self):
        self.assertEqual(Poll.objects.count(), 0)


class FixtureBleedThroughWithoutTransactionManagementTestCase(FixtureBleedThroughTestCase):  # noqa
    use_transaction_isolation = False


class NonFixtureBleedThroughWithoutTransactionManagementTestCase(NonFixtureBleedThroughTestCase):  # noqa
    use_transaction_isolation = False



from __future__ import with_statement

import os
import shutil
import signal
import socket
import subprocess
import sys
import tempfile
import time

from nosedjango.plugins.base_plugin import Plugin


class SphinxSearchPlugin(Plugin):
    """
    Plugin for configuring and running a sphinx search process for djangosphinx
    that's hooked up to a django test database.
    """
    name = 'django-sphinxsearch'
    searchd_port = 45798

    def __init__(self, *args, **kwargs):
        super(SphinxSearchPlugin, self).__init__(*args, **kwargs)

        self.tmp_sphinx_dir = None
        self.sphinx_config_tpl = None
        self._searchd = None

    def options(self, parser, env=None):
        """
        Sphinx config file that can optionally take the following python
        template string arguments:

        ``database_name``
        ``database_password``
        ``database_username``
        ``database_host``
        ``database_port``
        ``sphinx_search_data_dir``
        ``searchd_log_dir``
        """
        if env is None:
            env = os.environ
        parser.add_option(
            '--sphinx-config-tpl',
            help='Path to the Sphinx configuration file template.',
        )

        super(SphinxSearchPlugin, self).options(parser, env)

    def configure(self, options, config):
        if options.sphinx_config_tpl:
            self.sphinx_config_tpl = os.path.abspath(options.sphinx_config_tpl)

            # Create a directory for storing the configs, logs and index files
            self.tmp_sphinx_dir = tempfile.mkdtemp()

        super(SphinxSearchPlugin, self).configure(options, config)

    def startTest(self, test):
        from django.conf import settings
        from django.db import connection
        if 'django.db.backends.mysql' in connection.settings_dict['ENGINE']:
            # Using startTest instead of beforeTest so that we can be sure that
            # the fixtures were already loaded with nosedjango's beforeTest
            build_sphinx_index = getattr(test, 'build_sphinx_index', False)
            run_sphinx_searchd = getattr(test, 'run_sphinx_searchd', False)

            if run_sphinx_searchd:
                # Need to build the config

                # Update the DjangoSphinx client to use the proper port and
                # index
                settings.SPHINX_PORT = self.searchd_port
                from djangosphinx import models as dj_sphinx_models
                dj_sphinx_models.SPHINX_PORT = self.searchd_port

                # Generate the sphinx configuration file from the template
                sphinx_config_path = os.path.join(
                    self.tmp_sphinx_dir,
                    'sphinx.conf',
                )

                db_dict = connection.settings_dict
                with open(self.sphinx_config_tpl, 'r') as tpl_f:
                    context = {
                        'database_name': db_dict['NAME'],
                        'database_username': db_dict['USER'],
                        'database_password': db_dict['PASSWORD'],
                        'database_host': db_dict['HOST'],
                        'database_port': db_dict['PORT'],
                        'sphinx_search_data_dir': self.tmp_sphinx_dir,
                        'searchd_log_dir': self.tmp_sphinx_dir,
                    }
                    tpl = tpl_f.read()
                    output = tpl % context

                    with open(sphinx_config_path, 'w') as sphinx_conf_f:
                        sphinx_conf_f.write(output)
                        sphinx_conf_f.flush()

            if build_sphinx_index:
                self._build_sphinx_index(sphinx_config_path)
            if run_sphinx_searchd:
                self._start_searchd(sphinx_config_path)

    def afterTest(self, test):
        from django.db import connection
        if 'django.db.backends.mysql' in connection.settings_dict['ENGINE']:
            if getattr(test.context, 'run_sphinx_searchd', False):
                self._stop_searchd()

    def finalize(self, test):
        # Delete the temporary sphinx directory
        shutil.rmtree(self.tmp_sphinx_dir, ignore_errors=True)

    def _build_sphinx_index(self, config):
        indexer = subprocess.Popen(
            [
                'indexer',
                '--config',
                config,
                '--all',
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        if indexer.wait() != 0:
            print "Sphinx Indexing Problem"
            stdout, stderr = indexer.communicate()
            print "stdout: %s" % stdout
            print "stderr: %s" % stderr

    def _start_searchd(self, config):
        self._searchd = subprocess.Popen(
            ['searchd', '--config', config, '--console',
             '--port', str(self.searchd_port)],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        returned = self._searchd.poll()
        if returned is not None:
            print "Sphinx Search unavailable. searchd exited with code: %s" % returned  # noqa
            stdout, stderr = self._searchd.communicate()
            print "stdout: %s" % stdout
            print "stderr: %s" % stderr

        self._wait_for_connection(self.searchd_port)

    def _wait_for_connection(self, port):
        """
        Wait until we can make a socket connection to sphinx.
        """
        connected = False
        max_tries = 10
        num_tries = 0
        wait_time = 0.5
        while not connected or num_tries >= max_tries:
            time.sleep(wait_time)
            try:
                af = socket.AF_INET
                addr = ('127.0.0.1', port)
                sock = socket.socket(af, socket.SOCK_STREAM)
                sock.connect(addr)
            except socket.error:
                if sock:
                    sock.close()
                num_tries += 1
                continue
            connected = True

        if not connected:
            print >> sys.stderr, "Error connecting to sphinx searchd"

    def _stop_searchd(self):
        try:
            if not self._searchd.poll():
                os.kill(self._searchd.pid, signal.SIGKILL)
                self._searchd.wait()
        except AttributeError:
            print sys.stderr, "Error stopping sphinx search daemon"
